//---------------------------------------------------------------------------
// Sample UDS application to use with Radio SkyPipe
// 31 march 2006 -- dave803sw@gmail.com
//
// Free software
//
// A timer event fires every 1 second and updates fake data that is sent
// via TCP/IP to Skypipe.
//
// Any user hardware that generates data just needs to have it available
// for the SendData routine.  The routine will execute on a Poll from Skypipe
// or every time the timer event executes if the set to Push mode.  There is
// a radio button on the screen to set this mode.
//
#include <vcl.h>
#pragma hdrstop
#include <stdio.h>
#include <math.hpp>
#include <math.h>
#include <ScktComp.hpp>
#include <Sockets.hpp>

#define revision "(rev 0   31 MAR 2006)"
#include "SimpleUDS.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TForm1 *Form1;
TCustomWinSocket *Connection;
bool ServerConnected = false;
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
        : TForm(Owner)
{

  // When program starts enable server socket to watch for connection
  // requests on a particular port
  //
  TForm1::SckServer->Port = PortEdit->Text.ToInt();
  TForm1::SckServer->Active = true;
  TrafficMemo->Lines->Add("[LISTENING]");
  //
  //
  Form1->Caption = "SkyPipe UDS example     ";
  Form1->Caption = Form1->Caption.cat_sprintf("%s",revision);


}
//---------------------------------------------------------------------------
void __fastcall TForm1::SckServerAccept(TObject *Sender,
      TCustomWinSocket *Socket)
{
      TrafficMemo->Lines->Add("[Connected]");
      // set up a global TCustomWinSocket so I can talk to port
      // from other places
      Connection = Socket;

}
//---------------------------------------------------------------------------
void __fastcall TForm1::SckServerClientConnect(TObject *Sender,
      TCustomWinSocket *Socket)
{
if (ShowFlow->State)
	TrafficMemo->Lines->Add("[Server: OnClientConnect]");

int Last = SckServer->Socket->ActiveConnections - 1;
String tmp = String("Client[") + String(Last) + String("] IP address is: ") +
	SckServer->Socket->Connections[Last]->RemoteAddress;
TrafficMemo->Lines->Add(tmp);
}
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
void __fastcall TForm1::SckServerGetSocket(TObject *Sender, int Socket,
      TServerClientWinSocket *&ClientSocket)
{
if (ShowFlow->State)
	TrafficMemo->Lines->Add("[Server: OnGetSocket]");
}

//---------------------------------------------------------------------------
void __fastcall TForm1::SckServerListen(TObject *Sender,
      TCustomWinSocket *Socket)
{
if (ShowFlow->State)
	TrafficMemo->Lines->Add("[Server: OnListen]");
}

//---------------------------------------------------------------------------
void __fastcall TForm1::SckServerClientRead(TObject *Sender,
      TCustomWinSocket *Socket)
{
  AnsiString MsgBuff = "";

  if (ShowFlow->State)
	TrafficMemo->Lines->Add("[Server: OnClientRead]");

  // Figure out *which* Client just sent us a message...
  int Client;
  int Total = SckServer->Socket->ActiveConnections;

  for (Client = 0; Client < Total; Client++)
	if (SckServer->Socket->Connections[Client] == Socket)
		break;
  MsgBuff = Socket->ReceiveText();
  String Reply = String("Client[") + String(Client) + String("]: ") +  MsgBuff;
  if (ShowFlow->State) TrafficMemo->Lines->Add(Reply);
  //
  ProcesClientMessages(MsgBuff,Socket);
  //
}

//---------------------------------------------------------------------------
void __fastcall TForm1::SckServerClientWrite(TObject *Sender,
      TCustomWinSocket *Socket)
{
if (ShowFlow->State)
	TrafficMemo->Lines->Add("[Server: OnClientWrite]");
}

//---------------------------------------------------------------------------
void __fastcall TForm1::SckServerGetThread(TObject *Sender,
      TServerClientWinSocket *ClientSocket,
      TServerClientThread *&SocketThread)
{
if (ShowFlow->State)
	TrafficMemo->Lines->Add("[Server: OnGetThread]");

}

//---------------------------------------------------------------------------
void __fastcall TForm1::SckServerThreadEnd(TObject *Sender,
      TServerClientThread *Thread)
{
if (ShowFlow->State)
	TrafficMemo->Lines->Add("[Server: OnThreadEnd]");
}

//---------------------------------------------------------------------------
void __fastcall TForm1::SckServerThreadStart(TObject *Sender,
      TServerClientThread *Thread)
{
if (ShowFlow->State)
	TrafficMemo->Lines->Add("[Server: OnThreadStart]");
}


void __fastcall TForm1::Timer1Timer(TObject *Sender)
{

  //
  AnsiString s;
  double CH1, CH2, CH3, CH4, CH5, CH6, CH7, CH8;
  //
  // This block of code generates fake data
  //
  CH1 = RandG(0.0,.1);
  CH2 = RandG(1.0,.1);
  CH3 = RandG(2.0,.1);
  CH4 = RandG(3.0,.1);
  CH5 = RandG(4.0,.1);
  CH6 = RandG(5.0,.1);
  CH7 = RandG(6.0,.1);
  CH8 = RandG(7.0,.1);
  //
  // This block of code puts the data is edit boxes on the form
  //
  CH1Edit->Text =  s.sprintf("%6.4f", CH1);
  CH2Edit->Text =  s.sprintf("%6.4f", CH2);
  CH3Edit->Text =  s.sprintf("%6.4f", CH3);
  CH4Edit->Text =  s.sprintf("%6.4f", CH4);
  CH5Edit->Text =  s.sprintf("%6.4f", CH5);
  CH6Edit->Text =  s.sprintf("%6.4f", CH6);
  CH7Edit->Text =  s.sprintf("%6.4f", CH7);
  CH8Edit->Text =  s.sprintf("%6.4f", CH8);
  //
  // If UDS is pushing data, this will send it out
  //
  if (ServerConnected && !PollPush->ItemIndex) SendData(Connection);

}
//---------------------------------------------------------------------------
void __fastcall TForm1::ProcesClientMessages(AnsiString MsgBuff, TCustomWinSocket *Socket)
{
  AnsiString Message[10],s;
  int MsgNum = 0;
  char TempS;
  int NumChan;
  // process messages
  // is there a message in the buffer from the client?
  // parse message buffer to get individual commands
  //
  for (int i=0; i<10; i++) Message[i] = "";
  char* CharBuffer = new char[MsgBuff.Length() + 1];
  StrCopy(CharBuffer, MsgBuff.c_str());
  // Skypipe commands separated by FF.  See how many messages there are
  for (int i=0; i<MsgBuff.Length(); i++) {
        TempS = CharBuffer[i];
        if (TempS != '\xff') Message[MsgNum] = Message[MsgNum].cat_sprintf("%c",TempS);
        else {
                if (ShowFlow->State)TrafficMemo->Lines->Add
                        (s.sprintf("[Process Message: #%i  %s]",MsgNum,Message[MsgNum]));
                MsgNum++;
     }
  }
  //
  // now process all messages
  //
  for (int i=0; i<MsgNum; i++) {
    if (Message[i] == "INIT") {  // command from Skypipe to initialize UDS
        //
        // code or call can be added here to
        // initialize hardware if not done already
        //
        // Send ready command back to Skypipe
        s = s.sprintf("%s","^^1001\xff");  // means UDS ready
        Socket->SendText(s);
      if (ShowFlow->State)TrafficMemo->Lines->Add(s.sprintf("%s","[Server Send: 1001]"));
    }
    if (Message[i] ==  "POLL") {
      // initialize polling hardware
    }
    if (Message[i] ==  "PUSH") {
      // initialize push hardware
    }
    if (Message[i] ==  "GETC") {
      // skypipe requesting the number of channels
      // 20131=1,  20132=2, 20133=3, etc.
      //
      // How many channels are enabled?
      NumChan = CountChan();
      // Tell Skypipe
      s = s.sprintf("%s","^^2013%i\xff",NumChan); // five channels
      Socket->SendText(s);
      if (ShowFlow->State)TrafficMemo->Lines->Add(s.sprintf("[Server Send: %s]",s));
    }
    if (Message[i] ==  "STAT") {
      // Looking for UDS "ready" message
      // if (Frames->Text.ToInt > 1) then have data pipe ready
      // send back ready message
      s = s.sprintf("%s","^^1001\xff");
      Socket->SendText(s);
      if (ShowFlow->State)TrafficMemo->Lines->Add(s.sprintf("[Server Send: %s]","1001"));
      ServerConnected = true;
    }
    if (Message[i] ==  "GETD") {  // Skypipe is looking for some data
      // Send out the data
      SendData(Socket);
    }
    if (Message[i] ==  "PINH") {
      // value requested for printer port was high
    }
    if (Message[i] ==  "PINL") {
      // value requested for printer port was low
    }
    if (Message[i] ==  "PEAK") {
      //
    }
    if (Message[i] ==  "WAVG") {
      //
    }
  }

}
//---------------------------------------------------------------------------
int __fastcall TForm1::CountChan()
{
      int NumChan = 0;
      if (UDSCheckBox1->State) NumChan++;
      if (UDSCheckBox2->State) NumChan++;
      if (UDSCheckBox3->State) NumChan++;
      if (UDSCheckBox4->State) NumChan++;
      if (UDSCheckBox5->State) NumChan++;
      if (UDSCheckBox6->State) NumChan++;
      if (UDSCheckBox7->State) NumChan++;
      if (UDSCheckBox8->State) NumChan++;
  return  NumChan;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::SendData(TCustomWinSocket *Socket)
{
  AnsiString s;
      if (UDSCheckBox1->State) {
          Socket->SendText(s.sprintf("#0%f\xFF",CH1Edit->Text.ToDouble()));
          if (ShowFlow->State)TrafficMemo->Lines->Add(s.sprintf("[Server Send: #0 %f]",CH1Edit->Text.ToDouble()));
      }
      if (UDSCheckBox2->State) {
          Socket->SendText(s.sprintf("#1%f\xFF",CH2Edit->Text.ToDouble()));
          if (ShowFlow->State)TrafficMemo->Lines->Add(s.sprintf("[Server Send: #1 %f]",CH2Edit->Text.ToDouble()));
      }
      if (UDSCheckBox3->State) {
          Socket->SendText(s.sprintf("#2%f\xFF",CH3Edit->Text.ToDouble()));
          if (ShowFlow->State)TrafficMemo->Lines->Add(s.sprintf("[Server Send: #2 %f]",CH3Edit->Text.ToDouble()));
      }
      if (UDSCheckBox4->State) {
          Socket->SendText(s.sprintf("#3%f\xFF",CH4Edit->Text.ToDouble()));
          if (ShowFlow->State)TrafficMemo->Lines->Add(s.sprintf("[Server Send: #3 %f]",CH4Edit->Text.ToDouble()));
      }
      if (UDSCheckBox5->State) {
          Socket->SendText(s.sprintf("#4%f\xFF",CH5Edit->Text.ToDouble()));
          if (ShowFlow->State)TrafficMemo->Lines->Add(s.sprintf("[Server Send: #4 %f]",CH5Edit->Text.ToDouble()));
      }
      if (UDSCheckBox6->State) {
          Socket->SendText(s.sprintf("#5%f\xFF",CH6Edit->Text.ToDouble()));
          if (ShowFlow->State)TrafficMemo->Lines->Add(s.sprintf("[Server Send: #5 %f]",CH6Edit->Text.ToDouble()));
      }
      if (UDSCheckBox7->State) {
          Socket->SendText(s.sprintf("#6%f\xFF",CH7Edit->Text.ToDouble()));
          if (ShowFlow->State)TrafficMemo->Lines->Add(s.sprintf("[Server Send: #6 %f]",CH7Edit->Text.ToDouble()));
      }
      if (UDSCheckBox8->State) {
          Socket->SendText(s.sprintf("#7%f\xFF",CH8Edit->Text.ToDouble()));
          if (ShowFlow->State)TrafficMemo->Lines->Add(s.sprintf("[Server Send: #7 %f]",CH8Edit->Text.ToDouble()));
      }
      // send end of data character
      Socket->SendText(s.sprintf("^^3001\xFF"));
      if (ShowFlow->State)TrafficMemo->Lines->Add(s.sprintf("[Server Send: %s]","3001"));
}

void __fastcall TForm1::SckServerClientError(TObject *Sender,
      TCustomWinSocket *Socket, TErrorEvent ErrorEvent, int &ErrorCode)
{
   AnsiString error,s;
   // What caused the error??
   switch  (ErrorEvent)  {
         case eeGeneral:
                error = "eeGeneral";
                break;
         case eeSend:
                error = "eeSend";
                break;
         case eeReceive:
                error = "eeReceive";
                break;
         case eeConnect:
                error = "eeConnect";
                break;
         case eeDisconnect:
                error = "eeDisconnect";
                break;
         case eeAccept:
                error = "eeAccept";
                break;
         default:
                break;
   }
   if (ShowFlow->State) TrafficMemo->Lines->Add
                        (s.sprintf("[%s %i]",error,ErrorCode));
   //
   // clear the error for now
   ErrorCode = 0;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Button1Click(TObject *Sender)
{
  // To stop pushing data, i am resetting mode to pull
  PollPush->ItemIndex = 1;
  Button1->Visible = false;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::PollPushClick(TObject *Sender)
{
 if (PollPush->ItemIndex) Button1->Visible = false;
 else Button1->Visible = true;
}
//---------------------------------------------------------------------------

