Attribute VB_Name = "TimeRoutines"
' VB has some strong limitations on providing accurate times
' This module provides a more accurate way to establish the current time
' using the TimeMilliseconds routine and the Windows API.

' The CalcTime procedure allows you to take into account an offset for
' Universal Time using the Global variable UToffset.

Public UToffset As Double
'Calculate UToffset with:     UToffset = -1 * (TZ / 24)
' were TZ is your timezone in hours from GMT

'data structure which Windows uses to return the current System Time
Type SYSTEMTIME
        Year As Integer
        Month As Integer
        DayOfWeek As Integer
        Day As Integer
        Hour As Integer
        Minute As Integer
        Second As Integer
        Milliseconds As Integer
End Type
Public Declare Function SetSystemTime Lib "kernel32" _
   (lpSystemTime As SYSTEMTIME) As Long
Public Declare Function SetLocalTime Lib "kernel32" _
   (lpSystemTime As SYSTEMTIME) As Long
Declare Sub GetLocalTime Lib "kernel32" (lpSystemTime As SYSTEMTIME)

Private Type TIME_ZONE_INFORMATION
    Bias As Long
    StandardName(32) As Integer
    StandardDate As SYSTEMTIME
    StandardBias As Long
    DaylightName(32) As Integer
    DaylightDate As SYSTEMTIME
    DaylightBias As Long
End Type

Public Declare Function GetTimeZoneInformationAny Lib "kernel32" Alias "GetTimeZoneInformation" (Buffer As Any) As Long
Function MakeTimeCorrection(secs As Double, Optional nTimeZone As Variant) As Double
'Correct the clock by secs seconds
'return the measured correction made in seconds
Dim R As Double
Dim R2 As Double
Dim Lmil As Double
Dim Lsec As Double
Dim Lmin As Double
Dim Lhr As Double
Dim LdayPart As Double
Dim StartTime As Double
StartTime = TimeMilliseconds
R = secs
Dim Lps As SYSTEMTIME
GetLocalTime Lps
Lmil = CDbl(Lps.Milliseconds)
Lsec = CDbl(Lps.Second)
Lmin = CDbl(Lps.Minute)
LdayPart = CDbl(DateSerial(Lps.Year, Lps.Month, Lps.Day))
Lhr = CDbl(Lps.Hour)
R2 = Lmil / 1000 + Lsec + Lmin * 60 + Lhr * 3600
'corrected part of a day in seconds including tz correction
'because setdate function requires UT
R2 = (R2 - R)
If Not IsMissing(nTimeZone) Then
    R2 = R2 + (-1 * (nTimeZone / 24) * 86400)
End If
If R2 < 0 Then
    'day=day-1
    LdayPart = LdayPart - 1
    R2 = 86400 + R2
ElseIf R2 > 86400 Then
        'day=day+1
        LdayPart = LdayPart + 1
        R2 = R2 - 86400
End If
Lhr = Int(R2 / 3600)
R2 = R2 - Lhr * 3600
Lmin = Int(R2 / 60)
R2 = R2 - Lmin * 60
Lsec = Int(R2)
Lmil = Int((R2 - Lsec) * 1000)
Lps.Year = Year(LdayPart)
Lps.Month = Month(LdayPart)
Lps.Day = Day(LdayPart)
Lps.Hour = CInt(Lhr)
Lps.Minute = CInt(Lmin)
Lps.Second = CInt(Lsec)
Lps.Milliseconds = CInt(Lmil)
If IsMissing(nTimeZone) Then
    SetLocalTime Lps
Else
    SetSystemTime Lps
End If
MakeTimeCorrection = (StartTime - TimeMilliseconds) * 86400
        
End Function

Function SetTimeMilliseconds(newTime) As Double
  'returns corrected time
  'seems to have an error sometimes that probably comes from rounding of seconds
  'for precision use maketimecorrection
  
  Dim R As Long
  Dim DayPart As Double
  Dim StartTime As Double
  'StartTime = TimeMilliseconds
  DayPart = newTime - Int(newTime)
  Dim lpSystemTime As SYSTEMTIME
  With lpSystemTime
    .Year = Year(newTime)
    .Month = Month(newTime)
    .Day = Day(newTime)
    .Hour = Hour(newTime)
    .Minute = Minute(newTime)
    .Second = Second(newTime)
    .Milliseconds = Abs(DayPart - (Hour(newTime) / 24 + Minute(newTime) / 1440 + Second(newTime) / 86400)) * 86400000
  End With
  SetSystemTime lpSystemTime
SetTimeMilliseconds = TimeMilliseconds ' (TimeMilliseconds - StartTime) * 86400

End Function
Function TimeMilliseconds() As Double
  
  Dim lpSystemTime As SYSTEMTIME
  GetLocalTime lpSystemTime
  TimeMilliseconds = CDbl(DateSerial(lpSystemTime.Year, lpSystemTime.Month, lpSystemTime.Day)) + lpSystemTime.Hour / 24# + lpSystemTime.Minute / (1440#) + lpSystemTime.Second / (86400#) + lpSystemTime.Milliseconds / (86400000#)
  
End Function
Function MilliSecTimeStr(qTime) As String
'return a string with milliseconds displayed HH:nn:ss.sss
Dim R2 As Double
Dim Lmil As Double
Dim Lsec As Double
Dim Lmin As Double
Dim Lhr As Double

R2 = CDbl(qTime - Int(qTime)) * 86400 'only dealing with the part of a day
Lhr = Int(R2 / 3600)
R2 = R2 - Lhr * 3600
Lmin = Int(R2 / 60)
R2 = R2 - Lmin * 60
Lsec = Int(R2)
Lmil = (R2 - Lsec) * 1000
If Lmil >= 1000 Then
    Lmil = Lmil - 1000
    Lsec = Lsec + 1
    If Lsec > 59 Then
        Lsec = 0
        Lmin = Lmin + 1
        If Lmin > 59 Then
            Lhr = Lhr + 1
            Lmin = 0
            If Lhr > 23 Then Lhr = 0
        End If
    End If
End If
            
MilliSecTimeStr = Format(Lhr, "00") + ":" + Format(Lmin, "00") + ":" + Format(Lsec, "00") + "." + Format(Lmil, "000")


End Function

Function CalcTime() As Double

CalcTime = TimeMilliseconds + UToffset

End Function

Public Function GetTimeZone() As Double
Dim retval As Long
Dim Buffer(0 To 42) As Long
Const TIME_ZONE_ID_INVALID = &HFFFFFFFF
Const TIME_ZONE_ID_UNKNOWN = 0
Const TIME_ZONE_ID_STANDARD = 1
Const TIME_ZONE_ID_DAYLIGHT = 2
retval = GetTimeZoneInformationAny(Buffer(0))
Select Case retval
    Case TIME_ZONE_ID_INVALID
        GetTimeZone = 0
    Case TIME_ZONE_ID_STANDARD, TIME_ZONE_ID_UNKNOWN
        GetTimeZone = (Buffer(0) + Buffer(21)) / 60
        'or (tzinfo.bias+tzinfo.standardbias)/60
    Case TIME_ZONE_ID_DAYLIGHT
        GetTimeZone = (Buffer(0) + Buffer(42)) / 60
        'or (tzinfo.bias+tzinfo.Daylightbias)/60
    Case Else
        GetTimeZone = 0
End Select
GetTimeZone = -1 * GetTimeZone
End Function
